#!/usr/bin/env bash

function installAtsFiles() {
  mkdir -p /opt/ats/var/
  mkdir -p /opt/ats/var/log
  mkdir -p /opt/ats/var/log/trafficserver
  mkdir -p /opt/ats/var/trafficserver
  mkdir -p /var/log/polycube

  rood_dir=$(pwd)

  #mv "$old_dir/var/log" /opt/ats/var
  chown nobody:nobody -R /opt/ats/var/*

  # shellcheck disable=SC2154
  #  old_dir="/opt/ats.old.$(date +%F-%T)"
  #  mkdir -p "$old_dir"
  cp /opt/ats/bin/node /tmp/node
  for f in /opt/ats/{bin,lib,libexec,lib64,include}; do
    if [ -d "$f" ]; then
      rm "$f" -rf
    fi
  done
  #  mv /opt/ats/{bin,lib,libexec,lib64} "$old_dir" || {
  #    for f in /opt/ats/{bin,lib,libexec,lib64,include}; do
  #      if [ -d %f ]; then
  #        exit 23
  #      fi
  #    done
  #  }

  tar -axf ats.tar.xz

  cp -r bin lib libexec lib64 include /opt/ats || {
    # shellcheck disable=SC2034
    for f in /opt/ats/{bin,lib,libexec,lib64,include}; do
      if ! [ -d "$f" ]; then
        exit 24
      fi
    done
  }

  cp "/tmp/node" /opt/ats/bin

  if ! rpm -qa | grep libpcap &>/dev/null; then
    yum install libpcap -y
  fi

  if [ -d root ]; then
    cd root
    tar -axf root.tar.xz
    rm root.tar.xz
    cd -

    install_dir root
  fi

  cd $rood_dir
}

function installNodeServiceFile() {
  install -D --group=0 --owner=0 --mode=0641 node.service /lib/systemd/system/ || {
    exit 25
  }

  install -D --group=0 --owner=0 --mode=0641 trafficserver.service /lib/systemd/system/ || {
    exit 25
  }

  systemctl daemon-reload
  systemctl enable node.service
}

function installNodeSupervisorFile() {
  install -D --group=0 --owner=0 --mode=0641 node.conf /etc/supervisor/conf.d || {
    exit 26
  }

  supervisorctl update
}

function update_os_search_path() {
  ./updateKit file appendIfExists /etc/ld.so.conf /opt/ats/lib
  ./updateKit file appendIfExists /etc/ld.so.conf /opt/ats/lib64
  ./updateKit file appendIfExists /etc/ld.so.conf /opt/llvm/lib64

  ldconfig
}

function install_ip_resource() {
  local ip_resource_file="$1"
  #  local version=$(echo $ip_resource_file | grep -Eo '[0-9]+')

  cp "$ip_resource_file" /opt/ats/var/trafficserver

  eval "./updateKit toml set /opt/ats/etc/node/config.toml confOption.IPResourcePath '\"$ip_resource_file\"'"

  pkill -USR1 node
}

function set_kernel_opt() {
  local name=$1
  local value=$2

  ./updateKit file appendIfExists /etc/sysctl.conf $(printf "%s=%s" "$name" "$value")

  local origin_value=$(sysctl $name | cut -d = -f 2)

  if [ -z $origin_value ] || [ $origin_value = $value ]; then
    return
  fi

  sysctl -p
}

function if_load_kernel_mod() {
  local name=$1

  lsmod | awk '{print $1}' | grep -E "^${name}$" &>/dev/null
}

function install_bbr() {
  # 判断内核版本
  major_version=$(uname -r | cut -d . -f 1)
  if [ $major_version -lt 5 ]; then
    echo "内核版本低于5"

    return 81
  fi

  # 设置内核参数
  set_kernel_opt net.core.default_qdisc fq
  set_kernel_opt net.ipv4.tcp_congestion_control bbr

  # 安装bbr模块
  if if_load_kernel_mod tcp_tcc; then
    return
  fi

  cp tcp_tcc.ko /opt/ats/lib

  insmod /opt/ats/lib/tcp_tcc.ko
}

function install_dir() {
  dir="$1"

  files=$(find "$dir" -type f)

  for file in $files; do
    install -D "$file" "${file#$dir}"
  done
}

function take_ip_resource() {
  if [ $(find . -maxdepth 1 -name '*.dat' | wc -l) -ne 1 ]; then
    exit 82
  fi
  find . -maxdepth 1 -name '*.dat' | xargs basename
}
